@@ -91,7 +91,7 @@ struct SNP_Calling_Parameters{
 	HashTable * cigar_event_table;
 	char * rebuilt_command_line;
 
-	unsigned long long int all_mapped_bases;
+	srInt_64 all_mapped_bases;
 	unsigned int fisher_normalisation_target; 
 
 	double start_time;
@@ -568,7 +568,7 @@ void fishers_test_on_block(struct SNP_Calling_Parameters * parameters, float * s
 
 				//#warning " ===================== If a known SNP is at POI, the Fisher's p-value is only compared with the 'upper bound' of the p-value. ====================="
 				//#warning " ===================== '1 &&' is the switch ========================"
-				if(all_result_needed || ( 1 &&  Known_SNP_here && p_middle < parameters -> cutoff_upper_bound ) ||  ( p_middle < p_cutoff && flanking_matched*20>(flanking_matched+ flanking_unmatched )*16)) 
+				if(all_result_needed || ( 0 &&  Known_SNP_here && p_middle < parameters -> cutoff_upper_bound ) ||  ( p_middle < p_cutoff && flanking_matched*20>(flanking_matched+ flanking_unmatched )*16)) 
 					snp_fisher_raw [i] = p_middle;
 				else	snp_fisher_raw [i] = -999;
 
