@@ -31,7 +31,7 @@
 //#define MAX_EVENT_ENTRIES_PER_SITE 12 
 //
 #define EVENT_ENTRIES_INIT_SIZE (9) 
-#define MAX_EVENT_ENTRIES_PER_SITE (9) 
+#define MAX_EVENT_ENTRIES_PER_SITE 9
 #define CHRO_EVENT_TYPE_REMOVED 0
 #define CHRO_EVENT_TYPE_INDEL 8
 #define CHRO_EVENT_TYPE_LONG_INDEL 16 
