@@ -504,7 +504,11 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 				}
 				else if(strcmp("BCLinput", long_options[option_index].name)==0) 
 				{
-					global_context->config.is_BCL_input=1;
+					global_context->config.scRNA_input_mode=GENE_INPUT_BCL;
+				}
+				else if(strcmp("scRNA_FQinput", long_options[option_index].name)==0) 
+				{
+					global_context->config.scRNA_input_mode=GENE_INPUT_SCRNA_FASTQ;
 				}
 				else if(strcmp("rg-id", long_options[option_index].name)==0) 
 				{
