@@ -59,7 +59,7 @@ void fullscan_usage()
 void report_pos(unsigned int pos)
 {
 	char * chro_name;
-	unsigned int chro_pos;
+	int chro_pos;
 	locate_gene_position(pos, &_global_offsets, &chro_name, &chro_pos);
 	SUBREADprintf ("%s,%u\n", chro_name,chro_pos);
 }
@@ -133,7 +133,7 @@ void full_scan_read(char * index_name, char * read_str)
 		{
 			if(only_chro){
 				char * chro_name;
-				unsigned int chro_pos;
+				int chro_pos;
 				locate_gene_position(current_pos, &_global_offsets, &chro_name, &chro_pos);
 				if(strcmp(chro_name, only_chro)!=0)continue;
 			}
@@ -155,7 +155,7 @@ void full_scan_read(char * index_name, char * read_str)
 
 int main (int argc , char ** argv)
 {
-	char index_name [1200];
+	char index_name [MAX_FILE_NAME_LENGTH];
 	char read_str [1208];
 	char c;
 	int i;
@@ -166,7 +166,7 @@ int main (int argc , char ** argv)
 		switch(c)
 		{
 			case 'i':
-				strncpy(index_name,  optarg, 1199);
+				strncpy(index_name,  optarg, MAX_FILE_NAME_LENGTH-1);
 				break;
 			case 'm':
 				MIN_REPORTING_RATIO = atof(optarg);
