@@ -13,7 +13,7 @@ int main(int argc, char ** argv)
 		for(chro_offset = 0; chro_offset<500000; chro_offset++)
 		{
 			if(chro_offset % 70==0) puts("");
-			char nch_i = rand() % 4 ;
+			char nch_i = myrand_rand() % 4 ;
 			char nch = nch_i?(nch_i<2?'T':(nch_i < 3?'C':'G')):'A';
 			
 			putchar(nch);
